@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_LINUX.c,v 1.15 2006/07/19 19:17:28 martinp Exp $
+ *  @version \$Id: sysdep_LINUX.c,v 1.17 2007/01/03 09:31:01 martinp Exp $
  *
  *  @file
  */
@@ -120,7 +120,7 @@ int device_usage_sysdep(Info_T inf) {
   }
 
   inf->f_bsize=           usage.f_bsize;
-  inf->f_blocks=          usage.f_blocks * usage.f_frsize / usage.f_bsize;
+  inf->f_blocks=          usage.f_blocks;
   inf->f_blocksfree=      usage.f_bavail;
   inf->f_blocksfreetotal= usage.f_bfree;
   inf->f_files=           usage.f_files;
