@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -75,7 +75,7 @@ static void set_environment(void);
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: env.c,v 1.32 2006/04/27 20:16:03 martinp Exp $
+ *  @version \$Id: env.c,v 1.36 2007/01/03 09:31:01 martinp Exp $
  * 
  *  @file
  */
@@ -178,37 +178,27 @@ static void set_sandbox(void) {
  */
 static void set_environment(void) {
 
-  char cwd[MAXPATHLEN];
   struct passwd *pw;
   
   /* Get password struct */
   if ( ! (pw= getpwuid(geteuid())) ) {
-    
     LogError("%s: You don't exist. Go away.\n", prog);
     exit(1);
-    
   }
+  Run.Env.home= xstrdup(pw->pw_dir);
+  Run.Env.user= xstrdup(pw->pw_name);
   
   /* Get CWD */
-  if ( ! (getcwd(cwd, sizeof(cwd))) ) {
-    
-    LogError("%s: Cannot read current directory -- %s\n",
-      prog, STRERROR);
+  Run.Env.cwd= xcalloc(sizeof(char), MAXPATHLEN+1);
+  if ( ! (getcwd(Run.Env.cwd, MAXPATHLEN)) ) {
+    LogError("%s: Cannot read current directory -- %s\n", prog, STRERROR);
     exit(1);
-    
   }
   
   /*
    * Save and clear the file creation mask
    */
   Run.umask= umask(0);
-  
-  /*
-   * Initialize the runtime environment object
-   */
-  Run.Env.cwd=  xstrdup(cwd);
-  Run.Env.home= xstrdup(pw->pw_dir);
-  Run.Env.user= xstrdup(pw->pw_name);
-  
+
 }
 
