@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -54,7 +54,7 @@
  *
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * @author Martin Pala <martinp@tildeslash.com>
- * @version \$Id: event.c,v 1.62 2006/06/28 09:02:43 martinp Exp $
+ * @version \$Id: event.c,v 1.64 2007/01/03 09:31:01 martinp Exp $
  * @file
  */
 
@@ -564,39 +564,41 @@ void Event_queue_process() {
       {
         LogError("%s: Processing failed - cannot open the event file %s -- %s\n",
           prog, file_name, STRERROR);
-        goto error;
+        goto error1;
       }
 
       /* read event structure version */
-      if(!(version = File_readQueue(file, &size)) || size != sizeof(int))
-        goto error;
+      if(!(version = File_readQueue(file, &size)) || size != sizeof(int)) {
+        LogError("skipping %s - unknown data format\n",
+          file_name, *version);
+        goto error2;
+      }
       if(*version != EVENT_VERSION)
       {
         LogError("Aborting event %s - incompatible data format version %d\n",
           file_name, *version);
-        unlink(file_name);
-        goto error;
+        goto error2;
       }
 
       /* read event structure */
       if(!(e = File_readQueue(file, &size)) || size != sizeof(*e))
-        goto error;
+        goto error2;
 
       /* read source */
       if(!(e->source = File_readQueue(file, &size)))
-        goto error;
+        goto error3;
 
       /* read group */
       if(!(e->group = File_readQueue(file, &size)))
-        goto error;
+        goto error3;
 
       /* read message */
       if(!(e->message = File_readQueue(file, &size)))
-        goto error;
+        goto error3;
 
       /* read event action */
       if(!(action = File_readQueue(file, &size)) || size != sizeof(short))
-        goto error;
+        goto error3;
       a->id = *action;
       if(e->state == STATE_FAILED)
       {
@@ -662,15 +664,17 @@ void Event_queue_process() {
         unlink(file_name);
       }
 
-      error:
-      FREE(version);
-      FREE(action);
+      error3:
       FREE(e->source);
       FREE(e->group);
       FREE(e->message);
       FREE(e);
+      FREE(action);
+      error2:
+      FREE(version);
       fclose(file);
     }
+    error1:
     de = readdir(dir);
   }
   Run.handler_init = FALSE;
