@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -61,6 +61,8 @@ static void *thread_wrapper(void *arg);
 /* The HTTP Thread */
 static pthread_t thread;
 
+static volatile int running = FALSE;
+
 
 /**
  *  Facade functions for the cervlet sub-system. Start/Stop the monit
@@ -68,7 +70,7 @@ static pthread_t thread;
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: http.c,v 1.8 2006/04/27 20:16:03 martinp Exp $
+ *  @version \$Id: http.c,v 1.10 2007/01/10 00:04:19 martinp Exp $
  *
  *  @file
  */
@@ -115,6 +117,7 @@ void monit_http(int action) {
   switch(action) {
     
   case STOP_HTTP:
+    if(!running) break;
     LogInfo("Shutting down %s HTTP server\n", prog);
     stop_httpd();
     if( (status= pthread_join(thread, NULL)) != 0) {
@@ -122,6 +125,7 @@ void monit_http(int action) {
           prog, strerror(status));
     } else {
       LogInfo("%s HTTP server stopped\n", prog);
+      running = FALSE;
     }
     break;
 
@@ -133,6 +137,7 @@ void monit_http(int action) {
           prog, strerror(status));
     } else {
       LogInfo("%s HTTP server started\n", prog);
+      running = TRUE;
     }
     break;
 
