@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -146,7 +146,7 @@ static char *get_service_status_text(Service_T);
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: cervlet.c,v 1.210 2006/07/19 19:10:34 martinp Exp $
+ *  @version \$Id: cervlet.c,v 1.218 2007/02/07 23:02:58 martinp Exp $
  *
  *  @file
  */
@@ -219,7 +219,7 @@ static void doGet(HttpRequest req, HttpResponse res) {
 		 "You do not have sufficent privilegs to access this page");
     } else {
       do_wakeupcall();
-      send_redirect(res, HOME);
+      send_redirect(res, ".");
     }
   } else if(ACTION(PIXEL)) {
     printPixel(res);
@@ -261,14 +261,14 @@ static void do_home(HttpRequest req, HttpResponse res) {
     " <tr bgcolor=\"#BBDDFF\">"
     "  <td colspan=2 valign=\"top\" align=\"left\" bgcolor=\"#EFF7FF\" width=\"100%%\">"
     "  <br><h2 align=\"center\">Monit Service Manager</h2>"
-    "  <p align=\"center\">Monit is <a href='/_runtime'>running</a> on %s "
+    "  <p align=\"center\">Monit is <a href='_runtime'>running</a> on %s "
     "  with <i>uptime, %s</i> and monitoring:</p><br>"
     "  </td>"
     " </tr>"
     "</table>"
     "<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\" border=\"0\">"
     "  <tr valign=\"middle\" bgcolor=\"#6F6F6F\">"
-    "    <td><img src=\"/_pixel\" width=\"1\" height=\"1\" alt=\"\"></td>"
+    "    <td><img src=\"_pixel\" width=\"1\" height=\"1\" alt=\"\"></td>"
     "  </tr>"
     "</table>", Run.localhostname, uptime);
 
@@ -295,7 +295,7 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "monit " VERSION "</font></a></center><br><br>");
   out_print(res,
     "<ul>"
-    "<li>Copyright &copy; 2000-2006 by <a "
+    "<li>Copyright &copy; 2000-2007 by <a "
     "href=\"http://www.tildeslash.com/monit/who.php\">the monit "
     "project group</a>. All Rights Reserved.<br> </small><p>"
     "<li>Portions of this software are copyright &copy; 1995, 1996 "
@@ -303,7 +303,7 @@ static void do_about(HttpRequest req, HttpResponse res) {
   out_print(res,
    "<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\" border=\"0\">"
    " <tr bgcolor=\"#6F6F6F\">"
-   " <td valign=\"bottom\"><img src=\"/_pixel\" width=\"1\" height=\"1\" "
+   " <td valign=\"bottom\"><img src=\"_pixel\" width=\"1\" height=\"1\" "
    "alt=\"\"></td></tr></table>");
   out_print(res,
     "<p>This program is free software; you can redistribute it and/or "
@@ -319,7 +319,7 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "<a class=grey href=\"http://www.tildeslash.com/monit/donate.php\">"
     "donation</a> and support our continued work with monit.</b></i></center></p>");
   out_print(res,
-    "<p></br><center>[Click <a href=\"/\">here</a> to go back to monit]"
+    "<p></br><center>[Click <a href=\".\">here</a> to go back to monit]"
     "</center></p>");
 
   FOOT
@@ -478,22 +478,20 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
       "<font size=+1 color='#f0000'>Stop monit http server?</font>"
       "</td>"
       "<td align=right>"
-      "<form method=GET action=%s>"
+      "<form method=GET action=\"_stop\">"
       "<input type=submit value=Go style='font-size: 12pt'></font>"
       "</form>"
-      "</td>",
-      STOP);
+      "</td>");
 
     out_print(res,
       "<td>"
       "<font size=+1>Force validate now?</font>"
       "</td>"
       "<td align=right>"
-      "<form method=GET action=%s>"
+      "<form method=GET action=\"_validate\">"
       "<input type=submit value=Go style='font-size: 12pt'></font>"
       "</form>"
-      "</td>",
-      VALIDATE);
+      "</td>");
 
     if(Run.dolog && !Run.use_syslog) {
       out_print(res,
@@ -501,11 +499,10 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
       "<font size=+1>View monit logfile?</font>"
       "</td>"
       "<td align=right>"
-      "<form method=GET action=%s>"
+      "<form method=GET action=\"_viewlog\">"
       "<input type=submit value=Go style='font-size: 12pt'></font>"
       "</form>"
-      "</td>",
-      VIEWLOG);
+      "</td>");
     }
     out_print(res,
       "</tr>"
@@ -616,7 +613,7 @@ static void handle_action(HttpRequest req, HttpResponse res) {
         Run.doaction = TRUE;
       }
       do_wakeupcall();
-      send_redirect(res, req->url);
+      send_redirect(res, *req->url=='/'?req->url+1:req->url);
     }
   } else {
     not_found(req, res);
@@ -690,17 +687,33 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
 
     if(s->start) {
       int i= 0;
-      out_print(res, "<tr><td>Start program</td><td>");
-      while(s->start->arg[i])
-        out_print(res, "%s ", s->start->arg[i++]);
+      out_print(res, "<tr><td>Start program</td><td>'");
+      while(s->start->arg[i]) {
+        if(i) out_print(res, " ");
+        out_print(res, "%s", s->start->arg[i++]);
+      }
+      out_print(res, "'");
+      if(s->start->has_uid)
+        out_print(res, " as uid %d", s->start->uid);
+      if(s->start->has_gid)
+        out_print(res, " as gid %d", s->start->gid);
+      out_print(res, " timeout %d cycle(s)", s->start->timeout);
       out_print(res, "</td></tr>");
     }
 
     if(s->stop) {
       int i= 0;
-      out_print(res, "<tr><td>Stop program</td><td>");
-      while(s->stop->arg[i])
-        out_print(res, "%s ", s->stop->arg[i++]);
+      out_print(res, "<tr><td>Stop program</td><td>'");
+      while(s->stop->arg[i]) {
+        if(i) out_print(res, " ");
+        out_print(res, "%s", s->stop->arg[i++]);
+      }
+      out_print(res, "'");
+      if(s->stop->has_uid)
+        out_print(res, " as uid %d", s->stop->uid);
+      if(s->stop->has_gid)
+        out_print(res, " as gid %d", s->stop->gid);
+      out_print(res, " timeout %d cycle(s)", s->stop->timeout);
       out_print(res, "</td></tr>");
     }
 
@@ -714,8 +727,10 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
         "<tr><td>Timeout</td>"
         "<td>If %d restart within %d cycles then %s else if passed "
         "then %s</td></tr>",
-         s->to_start, s->to_cycle, actionnames[a->failed->id],
-         actionnames[a->passed->id]);
+         s->to_start,
+         s->to_cycle,
+         a->failed->description,
+         a->passed->description);
     }
 
     ctime_r(&s->collected, time);
@@ -818,7 +833,7 @@ static void do_home_system(HttpRequest req, HttpResponse res) {
   out_print(res,
     "</tr>"
     "<tr bgcolor=\"#EFEFEF\">"
-    "<td align=\"left\"><a href='/%s'>%s</a></td>"
+    "<td align=\"left\"><a href='%s'>%s</a></td>"
     "<td align=\"left\">%s</td>",
     s->name, s->name,
     status);
@@ -885,7 +900,7 @@ static void do_home_process(HttpRequest req, HttpResponse res) {
     status= get_service_status_html(s);
     out_print(res,
       "<tr %s>"
-      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td width=\"20%%\"><a href='%s'>%s</a></td>"
       "<td align=\"left\">%s</td>",
       on?"bgcolor=\"#EFEFEF\"":"",
       s->name, s->name,
@@ -965,7 +980,7 @@ static void do_home_device(HttpRequest req, HttpResponse res) {
     status= get_service_status_html(s);
     out_print(res,
       "<tr %s>"
-      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td width=\"20%%\"><a href='%s'>%s</a></td>"
       "<td align=\"left\">%s</td>",
       on?"bgcolor=\"#EFEFEF\"":"",
       s->name, s->name,
@@ -995,8 +1010,7 @@ static void do_home_device(HttpRequest req, HttpResponse res) {
       } else {
 
         out_print(res,
-	  "<td align=\"right\"><font color=red>not supported by filesystem"
-          "</font></td>");
+	  "<td align=\"right\">not supported by filesystem</td>");
 
       }
 
@@ -1046,7 +1060,7 @@ static void do_home_file(HttpRequest req, HttpResponse res) {
     status= get_service_status_html(s);
     out_print(res,
       "<tr %s>"
-      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td width=\"20%%\"><a href='%s'>%s</a></td>"
       "<td align=\"left\">%s</td>",
       on?"bgcolor=\"#EFEFEF\"":"",
       s->name, s->name,
@@ -1118,7 +1132,7 @@ static void do_home_fifo(HttpRequest req, HttpResponse res) {
     status= get_service_status_html(s);
     out_print(res,
       "<tr %s>"
-      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td width=\"20%%\"><a href='%s'>%s</a></td>"
       "<td align=\"left\">%s</td>",
       on?"bgcolor=\"#EFEFEF\"":"",
       s->name, s->name,
@@ -1187,7 +1201,7 @@ static void do_home_directory(HttpRequest req, HttpResponse res) {
     status= get_service_status_html(s);
     out_print(res,
       "<tr %s>"
-      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td width=\"20%%\"><a href='%s'>%s</a></td>"
       "<td align=\"left\">%s</td>",
       on?"bgcolor=\"#EFEFEF\"":"",
       s->name, s->name,
@@ -1256,7 +1270,7 @@ static void do_home_host(HttpRequest req, HttpResponse res) {
     status= get_service_status_html(s);
     out_print(res,
       "<tr %s>"
-      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td width=\"20%%\"><a href='%s'>%s</a></td>"
       "<td align=\"left\">%s</td>",
       on?"bgcolor=\"#EFEFEF\"":"",
       s->name, s->name,
@@ -1325,10 +1339,10 @@ static void print_alerts(HttpResponse res, Mail_T s) {
 	      "<td>%s</td></tr>", r->to?r->to:"");
     out_print(res, "<tr><td>Alert on</td><td>");
 
-    if(r->events == (~((unsigned int)0))) {
-      
+    if(r->events == EVENT_NULL) {
+      out_print(res, "No events");
+    } else if(r->events == EVENT_ALL) {
       out_print(res, "All events");
-
     } else {
       if(IS_EVENT_SET(r->events, EVENT_CHANGED))
 	  out_print(res, "Change ");
@@ -1394,27 +1408,27 @@ static void print_buttons(HttpRequest req, HttpResponse res, Service_T s) {
   /* Start program */
   if(s->start)
       out_print(res, 
-		"<td><form method=GET action=/%s>"
+		"<td><form method=GET action=%s>"
 		"<input type=hidden value='start' name=action>"
 		"<input type=submit value='Start service' style='font-size: "
 		"12pt'></form></td>", s->name);
   /* Stop program */
   if(s->stop)
       out_print(res, 
-		"<td><form method=GET action=/%s>"
+		"<td><form method=GET action=%s>"
 		"<input type=hidden value='stop' name=action>"
 		"<input type=submit value='Stop service' style='font-size: "
 		"12pt'></form></td>", s->name);
   /* Restart program */
   if(s->start && s->stop)
       out_print(res, 
-		"<td><form method=GET action=/%s>"
+		"<td><form method=GET action=%s>"
 		"<input type=hidden value='restart' name=action>"
 		"<input type=submit value='Restart service' style='font-size: "
 		"12pt'></form></td>", s->name);
   /* (un)monitor */
   out_print(res, 
-	    "<td><form method=GET action=/%s>"
+	    "<td><form method=GET action=%s>"
 	    "<input type=hidden value='%s' name=action>"
 	    "<input type=submit value='%s' style='font-size: 12pt'>"
 	    "</form></td></tr></table>",
@@ -1448,8 +1462,8 @@ static void print_service_rules_port(HttpResponse res, Service_T s) {
           p->hostname, p->port, p->request?p->request:"",
           p->protocol->name, Util_portTypeDescription(p),
           p->timeout,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
         if(p->SSL.certmd5 != NULL)
           out_print(res,
             "<tr><td>Server certificate md5 sum</td><td>%s</td></tr>",
@@ -1460,8 +1474,8 @@ static void print_service_rules_port(HttpResponse res, Service_T s) {
           "If failed %s [%s] with timeout %ds %s then %s else if passed %s then %s"
           "</td></tr>",
           p->pathname, p->protocol->name, p->timeout,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
       }
     }
   }
@@ -1488,8 +1502,8 @@ static void print_service_rules_icmp(HttpResponse res, Service_T s) {
         "If failed %s count %d with timeout %d seconds %s then %s else if passed %s then %s"
         "</td></tr>",
         icmpnames[i->type], i->count, i->timeout,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
     }
   }
 }
@@ -1509,8 +1523,8 @@ static void print_service_rules_perm(HttpResponse res, Service_T s) {
     out_print(res, "<tr><td>Associated permission</td>"
       "<td>If failed %o %s then %s else if passed %s then %s</td></tr>",
       s->perm->perm,
-      ratio1, actionnames[a->failed->id],
-      ratio2, actionnames[a->passed->id]);
+      ratio1, a->failed->description,
+      ratio2, a->passed->description);
 
   }
 }
@@ -1530,8 +1544,8 @@ static void print_service_rules_uid(HttpResponse res, Service_T s) {
     out_print(res, "<tr><td>Associated UID</td>"
       "<td>If failed %d %s then %s else if passed %s then %s</td></tr>",
       (int)s->uid->uid,
-      ratio1, actionnames[a->failed->id],
-      ratio2, actionnames[a->passed->id]);
+      ratio1, a->failed->description,
+      ratio2, a->passed->description);
   }
 }
 
@@ -1550,8 +1564,8 @@ static void print_service_rules_gid(HttpResponse res, Service_T s) {
     out_print(res, "<tr><td>Associated GID</td>"
       "<td>If failed %d %s then %s else if passed %s then %s</td></tr>",
       (int)s->gid->gid,
-      ratio1, actionnames[a->failed->id],
-      ratio2, actionnames[a->passed->id]);
+      ratio1, a->failed->description,
+      ratio2, a->passed->description);
   }
 }
 
@@ -1576,7 +1590,7 @@ static void print_service_rules_timestamp(HttpResponse res, Service_T s) {
 
         out_print(res,
           "<tr><td>Associated timestamp</td><td>If changed %s then %s</td></tr>",
-          ratio1, actionnames[a->failed->id]);
+          ratio1, a->failed->description);
 
       } else {
 
@@ -1584,8 +1598,8 @@ static void print_service_rules_timestamp(HttpResponse res, Service_T s) {
           "<tr><td>Associated timestamp</td>"
           "<td>If %s %d second(s) %s then %s else if passed %s then %s</td></tr>",
           operatornames[t->operator], t->time,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
       }
     }
   }
@@ -1600,7 +1614,7 @@ static void print_service_rules_device(HttpResponse res, Service_T s) {
 
     out_print(res, "<tr><td>Filesystem flags</td><td>If changed %s then %s</td></tr>\n",
       Util_getEventratio(s->action_FSFLAG->failed, ratio1),
-      actionnames[s->action_FSFLAG->failed->id]);
+      s->action_FSFLAG->failed->description);
   }
 
   if(s->devicelist) {
@@ -1625,16 +1639,16 @@ static void print_service_rules_device(HttpResponse res, Service_T s) {
 	    "then %s else if passed %s then %s</td></tr>",
 	    operatornames[dl->operator],
 	    dl->limit_absolute,
-            ratio1, actionnames[a->failed->id],
-            ratio2, actionnames[a->passed->id]);
+            ratio1, a->failed->description,
+            ratio2, a->passed->description);
         } else {
           out_print(res,
 	    "<tr><td>Inodes usage limit</td><td>If %s %.1f%% %s "
 	    "then %s else if passed %s then %s</td></tr>",
 	    operatornames[dl->operator],
 	    dl->limit_percent/10.,
-            ratio1, actionnames[a->failed->id],
-            ratio2, actionnames[a->passed->id]);
+            ratio1, a->failed->description,
+            ratio2, a->passed->description);
         }
 	  
       } else if(dl->resource == RESOURCE_ID_SPACE) {
@@ -1645,16 +1659,16 @@ static void print_service_rules_device(HttpResponse res, Service_T s) {
 	    "then %s else if passed %s then %s</td></tr>",
             operatornames[dl->operator],
 	    dl->limit_absolute,
-            ratio1, actionnames[a->failed->id],
-            ratio2, actionnames[a->passed->id]);
+            ratio1, a->failed->description,
+            ratio2, a->passed->description);
         } else {
           out_print(res,
 	    "<tr><td>Space usage limit</td><td>If %s %.1f%% %s "
 	    "then %s else if passed %s then %s</td></tr>",
             operatornames[dl->operator],
 	    dl->limit_percent/10.,
-            ratio1, actionnames[a->failed->id],
-            ratio2, actionnames[a->passed->id]);
+            ratio1, a->failed->description,
+            ratio2, a->passed->description);
         }
       }
     }
@@ -1682,7 +1696,7 @@ static void print_service_rules_size(HttpResponse res, Service_T s) {
 
         out_print(res,
           "<tr><td>Associated size</td><td>If changed %s then %s</td></tr>",
-          ratio1, actionnames[a->failed->id]);
+          ratio1, a->failed->description);
 
       } else {
 
@@ -1690,8 +1704,8 @@ static void print_service_rules_size(HttpResponse res, Service_T s) {
           "<tr><td>Associated size</td>"
           "<td>If %s %llu byte(s) %s then %s else if passed %s then %s</td></tr>",
           operatornames[sl->operator], sl->size,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 
       }
     }
@@ -1716,7 +1730,7 @@ static void print_service_rules_match(HttpResponse res, Service_T s) {
                 "<tr><td>Associated regex</td><td>If %s match "
                 "\"%s\" %s then %s</td></tr>",
                 ml->not?"not ":"", ml->match_string,
-                ratio1, actionnames[a->failed->id]);
+                ratio1, a->failed->description);
       
     }
   }
@@ -1740,15 +1754,15 @@ static void print_service_rules_checksum(HttpResponse res, Service_T s) {
       out_print(res,
         "<tr><td>Associated checksum</td><td>if changed %s %s then %s</td></tr>",
         checksumnames[cs->type],
-        ratio1, actionnames[a->failed->id]);
+        ratio1, a->failed->description);
 
     } else {
 
       out_print(res, "<tr><td>Associated checksum</td><td>"
         "if failed %s(%s) %s then %s else if passed %s then %s</td></tr>",
         cs->hash, checksumnames[cs->type],
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
 
     }
   }
@@ -1763,10 +1777,10 @@ static void print_service_rules_process(HttpResponse res, Service_T s) {
 
     out_print(res, "<tr><td>Pid</td><td>If changed %s then %s</td></tr>\n",
       Util_getEventratio(s->action_PID->failed, ratio1),
-      actionnames[s->action_PID->failed->id]);
+      s->action_PID->failed->description);
     out_print(res, "<tr><td>Ppid</td><td>If changed %s then %s</td></tr>\n",
       Util_getEventratio(s->action_PPID->failed, ratio1),
-      actionnames[s->action_PPID->failed->id]);
+      s->action_PPID->failed->description);
   }
 }
 
@@ -1796,8 +1810,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_CPUUSER: 
@@ -1807,8 +1821,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_CPUSYSTEM: 
@@ -1818,8 +1832,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_CPUWAIT: 
@@ -1829,8 +1843,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_MEM_PERCENT: 
@@ -1840,8 +1854,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_MEM_KBYTE: 
@@ -1851,8 +1865,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_LOAD1: 
@@ -1862,8 +1876,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "else if passed %s then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_LOAD5: 
@@ -1873,8 +1887,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_LOAD15: 
@@ -1884,8 +1898,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "then %s</td></tr>", 
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_CHILDREN:
@@ -1895,8 +1909,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "then %s</td></tr>",
 	  operatornames[q->operator],
 	  q->limit,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_TOTAL_MEM_KBYTE:
@@ -1906,8 +1920,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "</td></tr>",
 	  operatornames[q->operator],
 	  q->limit,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       case RESOURCE_ID_TOTAL_MEM_PERCENT:
@@ -1917,8 +1931,8 @@ static void print_service_rules_resource(HttpResponse res, Service_T s) {
           "</td></tr>",
 	  operatornames[q->operator],
 	  q->limit/10.0,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	break;
 	  
       }    
