@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -107,7 +107,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala <martinp@tildeslash.com>
  *
- *  @version \$Id: engine.c,v 1.73 2006/06/29 21:08:32 martinp Exp $
+ *  @version \$Id: engine.c,v 1.77 2007/01/03 09:31:02 martinp Exp $
  *
  *  @file 
  */
@@ -229,72 +229,50 @@ void stop_httpd() {
  */
 int add_host_allow(char *name) {
 
-  struct hostent *hp;
-  struct in_addr *inp;
+  struct addrinfo hints;
+  struct addrinfo *res;
+  struct addrinfo *_res;
 
   ASSERT(name);
 
-  if(! (hp= gethostbyname(name))) {
-    
-    LogError("%s: Error: hostname did not resolve '%s'.\n",
-          prog, name); 
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = PF_INET; /* we support just IPv4 currently */
+
+  if(getaddrinfo(name, NULL, &hints, &res) != 0)
     return FALSE;
-    
-  } else  {
-    
-    while(*hp->h_addr_list) {
 
-      HostsAllow h;
+  for(_res = res; _res; _res = _res->ai_next) {
+    if(_res->ai_family == AF_INET) {
+       HostsAllow h;
+       struct sockaddr_in *sin = (struct sockaddr_in *)_res->ai_addr;
 
-      NEW(h);      
-
-      inp = (struct in_addr *) *hp->h_addr_list++;
-
-      h->network= inp->s_addr;
-      h->mask=    0xffffffff;
-      
-      LOCK(hostlist_mutex)
-	
-      if(hostlist) {
-      
-        HostsAllow p, n;
-      
-        for(n= p= hostlist; p; n= p, p= p->next) {
-
-          if((p->network == h->network) && ((p->mask == h->mask))) {
-            DEBUG("%s: Debug: Skipping redundant host '%s' (%s).\n",
-                  prog, name, inet_ntoa(*inp)); 
-
-            destroy_host_allow(h);
-            goto done;
-	  
-          }
-	
-        }
-
-        DEBUG("%s: Debug: Adding host allow '%s' (%s).\n",
-              prog, name, inet_ntoa(*inp)); 
+       NEW(h);      
+       memcpy(&h->network, &sin->sin_addr, 4);
+       h->mask=    0xffffffff;
+       LOCK(hostlist_mutex)
+       if(hostlist) {
+         HostsAllow p, n;
+         for(n= p= hostlist; p; n= p, p= p->next) {
+           if((p->network == h->network) && ((p->mask == h->mask))) {
+             DEBUG("%s: Debug: Skipping redundant host '%s'\n", prog, name); 
+             destroy_host_allow(h);
+             goto done;
+           }
+         }
+         DEBUG("%s: Debug: Adding host allow '%s'\n", prog, name); 
+         n->next= h;
+       } else {
+         DEBUG("%s: Debug: Adding host allow '%s'\n", prog, name);
         
-        n->next= h;
-      
-      } else {
-      
-        DEBUG("%s: Debug: Adding host allow '%s' (%s).\n",
-              prog, name, inet_ntoa(*inp));
-        
-        hostlist= h;
-      
-      }
-      
-      done:
-      END_LOCK;
-
+         hostlist= h;
+       }
+       done:
+       END_LOCK;
     }
-
   }
 
+  freeaddrinfo(res);
   return TRUE;
-
 }
 
 /**
