@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -248,6 +248,7 @@ typedef struct mycommand {
   uid_t uid;         /**< The user id to switch to when running this Command */
   int   has_gid;          /**< TRUE if a new gid is defined for this Command */
   gid_t gid;        /**< The group id to switch to when running this Command */
+  unsigned timeout;      /**< Max cycles which we wait for method to execute */
 } *Command_T;
 
 
@@ -257,6 +258,7 @@ typedef struct myaction {
     Command_T exec;                    /**< Optional command to be executed  */
     unsigned  count;           /**< Event count needed to trigger the action */
     unsigned  cycles;    /**< Cycles during which count limit can be reached */
+    char description[STRLEN];                 /**< Action description buffer */
 } *Action_T;
 
 
