@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -134,7 +134,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.c,v 1.72 2006/06/30 16:41:51 martinp Exp $
+ *  @version \$Id: net.c,v 1.75 2007/01/03 09:31:01 martinp Exp $
  *
  *  @file
  */
@@ -157,24 +157,18 @@ static unsigned short checksum_ip(unsigned char *, int);
  */
 int check_host(const char *hostname) {
 
-#ifdef HAVE_GETADDRINFO        
+  struct addrinfo hints;
   struct addrinfo *res;
-#else
-  struct hostent *hp;
-#endif  
 
   ASSERT(hostname);
 
-#ifdef HAVE_GETADDRINFO        
-  if(getaddrinfo(hostname, NULL, NULL, &res)!=0) {
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = PF_INET; /* we support just IPv4 currently */
+
+  if(getaddrinfo(hostname, NULL, &hints, &res) != 0)
     return FALSE;
-  }
+
   freeaddrinfo(res);
-#else
-  if((hp = gethostbyname(hostname)) == NULL) {
-          return FALSE;
-  }
-#endif  
 
   return TRUE;
 
