@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -69,7 +69,7 @@
  *  @author Rory Toma <rory@digeo.com>
  *  @author Martin Pala <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_DARWIN.c,v 1.28 2006/06/21 18:59:54 martinp Exp $
+ *  @version \$Id: sysdep_DARWIN.c,v 1.31 2007/01/03 09:31:02 martinp Exp $
  *
  *  @file
  */
@@ -175,9 +175,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
 
     pt[i].pid       = pinfo[i].kp_proc.p_pid;
     pt[i].ppid      = pinfo[i].kp_eproc.e_ppid;
-    pt[i].cputime   = (long)((pinfo[i].kp_proc.p_rtime.tv_sec * 10) +
-                             (pinfo[i].kp_proc.p_rtime.tv_usec / 100000)
-                            );
+
     if(pinfo[i].kp_proc.p_stat == SZOMB)
     {
       pt[i].status_flag |= PROCESS_ZOMBIE;
@@ -185,11 +183,32 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
     pt[i].time = get_float_time();
 
     if(task_for_pid(mach_task_self(), pt[i].pid, &task) == KERN_SUCCESS) {
-      task_basic_info_data_t info;
-      mach_msg_type_number_t count = TASK_BASIC_INFO_COUNT;
+      mach_msg_type_number_t   count;
+      task_basic_info_data_t   taskinfo;
+      thread_array_t           threadtable;
+      unsigned int             threadtable_size;
+      thread_basic_info_t      threadinfo;
+      thread_basic_info_data_t threadinfo_data;
 
-      if(task_info(task, TASK_BASIC_INFO, (task_info_t)&info, &count) == KERN_SUCCESS) {
-        pt[i].mem_kbyte = (unsigned long)(info.resident_size / 1024);
+      count = TASK_BASIC_INFO_COUNT;
+      if(task_info(task, TASK_BASIC_INFO, (task_info_t)&taskinfo, &count) == KERN_SUCCESS) {
+        pt[i].mem_kbyte = (unsigned long)(taskinfo.resident_size / 1024);
+        pt[i].cputime = (long)((taskinfo.user_time.seconds      + taskinfo.system_time.seconds)      * 10     +
+                               (taskinfo.user_time.microseconds + taskinfo.system_time.microseconds) / 100000);
+      }
+      if(task_threads(task, &threadtable, &threadtable_size) == KERN_SUCCESS) {
+        int                      j;
+
+        threadinfo = &threadinfo_data;
+        for (j = 0; j < threadtable_size; j++) {
+                count = THREAD_BASIC_INFO_COUNT;
+                if (thread_info(threadtable[j], THREAD_BASIC_INFO, (thread_info_t)threadinfo, &count) == KERN_SUCCESS) {
+                        if((threadinfo->flags & TH_FLAGS_IDLE) == 0) {
+                          pt[i].cputime += (long)((threadinfo->user_time.seconds      + threadinfo->system_time.seconds)      * 10     +
+                                                  (threadinfo->user_time.microseconds + threadinfo->system_time.microseconds) / 100000);
+                        }
+                }
+        }
       }
     }
   }
