@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -119,7 +119,7 @@ static int is_url_unsafe(unsigned char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.186 2006/07/19 17:33:37 martinp Exp $
+ *  @version \$Id: util.c,v 1.192 2007/02/02 09:57:12 martinp Exp $
  *  @file
  */
 
@@ -701,17 +701,33 @@ void Util_printService(Service_T s) {
   if(s->start) {
     int i = 0;
 
-    printf(" %-20s =", "Start program");
-    while(s->start->arg[i])
-      printf(" %s", s->start->arg[i++]);
+    printf(" %-20s = '", "Start program");
+    while(s->start->arg[i]) {
+      if(i) printf(" ");
+      printf("%s", s->start->arg[i++]);
+    }
+    printf("'");
+    if(s->start->has_uid)
+      printf(" as uid %d", s->start->uid);
+    if(s->start->has_gid)
+      printf(" as gid %d", s->start->gid);
+    printf(" timeout %d cycle(s)", s->start->timeout);
     printf("\n");
   }
   if(s->stop) {
     int i = 0;
 
-    printf(" %-20s =", "Stop program");
-    while(s->stop->arg[i])
-      printf(" %s", s->stop->arg[i++]);
+    printf(" %-20s = '", "Stop program");
+    while(s->stop->arg[i]) {
+      if(i) printf(" ");
+      printf("%s", s->stop->arg[i++]);
+    }
+    printf("'");
+    if(s->stop->has_uid)
+      printf(" as uid %d", s->stop->uid);
+    if(s->stop->has_gid)
+      printf(" as gid %d", s->stop->gid);
+    printf(" timeout %d cycle(s)", s->stop->timeout);
     printf("\n");
   }
 
@@ -723,18 +739,19 @@ void Util_printService(Service_T s) {
     printf(" %-20s = if changed %s then %s\n",
       "Pid",
       Util_getEventratio(s->action_PID->failed, ratio1),
-      actionnames[s->action_PID->failed->id]);
+      s->action_PID->failed->description);
+
     printf(" %-20s = if changed %s then %s\n",
       "Ppid",
       Util_getEventratio(s->action_PPID->failed, ratio1),
-      actionnames[s->action_PPID->failed->id]);
+      s->action_PPID->failed->description);
   }
 
   if(s->type == TYPE_DEVICE) {
     printf(" %-20s = if changed %s then %s\n",
       "Filesystem flags",
       Util_getEventratio(s->action_FSFLAG->failed, ratio1),
-      actionnames[s->action_FSFLAG->failed->id]);
+      s->action_FSFLAG->failed->description);
   }
 
   if(s->checksum && s->checksum->action) {
@@ -747,12 +764,12 @@ void Util_printService(Service_T s) {
     if(cs->test_changes)
       printf(" %-20s = if changed %s %s then %s\n",
         "Checksum", checksumnames[cs->type],
-        ratio1, actionnames[a->failed->id]);
+        ratio1, a->failed->description);
     else
       printf(" %-20s = if failed %s(%s) %s then %s else if passed %s then %s\n",
         "Checksum", cs->hash, checksumnames[cs->type],
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
   }
   
   if(s->perm && s->perm->action) {
@@ -763,8 +780,8 @@ void Util_printService(Service_T s) {
 
     printf(" %-20s = if failed %o %s then %s else if passed %s then %s\n",
       "Permission", s->perm->perm,
-      ratio1, actionnames[a->failed->id],
-      ratio2, actionnames[a->passed->id]);
+      ratio1, a->failed->description,
+      ratio2, a->passed->description);
   }
 
   if(s->uid && s->uid->action) {
@@ -775,8 +792,8 @@ void Util_printService(Service_T s) {
 
     printf(" %-20s = if failed %d %s then %s else if passed %s then %s\n",
       "UID", (int)s->uid->uid,
-      ratio1, actionnames[a->failed->id],
-      ratio2, actionnames[a->passed->id]);
+      ratio1, a->failed->description,
+      ratio2, a->passed->description);
   }
 
   if(s->gid && s->gid->action) {
@@ -787,8 +804,8 @@ void Util_printService(Service_T s) {
 
     printf(" %-20s = if failed %d %s then %s else if passed %s then %s\n",
       "GID", (int)s->gid->gid,
-      ratio1, actionnames[a->failed->id],
-      ratio2, actionnames[a->passed->id]);
+      ratio1, a->failed->description,
+      ratio2, a->passed->description);
   }
 
   if(s->icmplist)
@@ -801,8 +818,8 @@ void Util_printService(Service_T s) {
       printf(" %-20s = if failed %s count %d with timeout %d seconds %s then %s "
         "else if passed %s then %s\n",
         "ICMP", icmpnames[i->type], i->count, i->timeout,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
     }
 
   if(s->portlist) {
@@ -820,8 +837,8 @@ void Util_printService(Service_T s) {
 	     "Port", n->hostname, n->port, n->request?n->request:"",
              n->protocol->name, Util_portTypeDescription(n),
              n->timeout,
-             ratio1, actionnames[a->failed->id],
-             ratio2, actionnames[a->passed->id]);
+             ratio1, a->failed->description,
+             ratio2, a->passed->description);
 	  
 	   if(n->SSL.certmd5 != NULL)
 	     printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
@@ -831,8 +848,8 @@ void Util_printService(Service_T s) {
         printf(" %-20s = if failed %s [protocol %s] with timeout %d seconds "
           "%s then %s else if passed %s then %s\n",
           "Unix Socket", n->pathname, n->protocol->name, n->timeout,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
 	
       }
       
@@ -849,12 +866,12 @@ void Util_printService(Service_T s) {
     if(t->test_changes)
       printf(" %-20s = if changed %s then %s\n",
         "Timestamp",
-        ratio1, actionnames[a->failed->id]);
+        ratio1, a->failed->description);
     else
       printf(" %-20s = if %s %d second(s) %s then %s else if passed %s then %s\n",
         "Timestamp", operatornames[t->operator], t->time,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
     
   }
 
@@ -867,12 +884,12 @@ void Util_printService(Service_T s) {
     if(sl->test_changes)
       printf(" %-20s = if changed %s then %s\n",
         "Size",
-        ratio1, actionnames[a->failed->id]);
+        ratio1, a->failed->description);
     else
       printf(" %-20s = if %s %llu byte(s) %s then %s else if passed %s then %s\n",
         "Size", operatornames[sl->operator], sl->size,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
     
   }
 
@@ -883,7 +900,7 @@ void Util_printService(Service_T s) {
 
     printf(" %-20s = if%s match \"%s\" %s then %s\n",
            "Regex", ml->not?" not":"", ml->match_string,
-           ratio1, actionnames[a->failed->id]);
+           ratio1, a->failed->description);
     
   }
   
@@ -900,15 +917,15 @@ void Util_printService(Service_T s) {
           "Inodes usage limit",
           operatornames[dl->operator],
           dl->limit_absolute,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
       } else {
         printf(" %-20s = if %s %.1f%% %s then %s else if passed %s then %s\n",
           "Inodes usage limit",
           operatornames[dl->operator],
           dl->limit_percent/10.,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
       }
       
     } else if(dl->resource == RESOURCE_ID_SPACE) {
@@ -918,15 +935,15 @@ void Util_printService(Service_T s) {
           "Space usage limit",
           operatornames[dl->operator],
           dl->limit_absolute,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
       } else {
         printf(" %-20s = if %s %.1f%% %s then %s else if passed %s then %s\n",
           "Space usage limit",
           operatornames[dl->operator],
           dl->limit_percent/10.,
-          ratio1, actionnames[a->failed->id],
-          ratio2, actionnames[a->passed->id]);
+          ratio1, a->failed->description,
+          ratio2, a->passed->description);
       }
 
     }
@@ -947,8 +964,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "CPU usage limit", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_CPUUSER: 
@@ -957,8 +974,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "CPU user limit", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_CPUSYSTEM: 
@@ -967,8 +984,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "CPU system limit", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_CPUWAIT: 
@@ -977,8 +994,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "CPU wait limit", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_MEM_PERCENT: 
@@ -987,8 +1004,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "Memory usage limit", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_MEM_KBYTE: 
@@ -997,8 +1014,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "Memory amount limit", 
         operatornames[q->operator], q->limit,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_LOAD1: 
@@ -1007,8 +1024,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "Load avg. (1min)", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_LOAD5: 
@@ -1017,8 +1034,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "Load avg. (5min)", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_LOAD15: 
@@ -1027,8 +1044,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n", 
         "Load avg. (15min)", 
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_CHILDREN:
@@ -1036,8 +1053,8 @@ void Util_printService(Service_T s) {
       printf(" %-20s = If %s %ld %s then %s "
         "else if passed %s then %s\n",
         "Children", operatornames[q->operator], q->limit,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_TOTAL_MEM_KBYTE:
@@ -1046,8 +1063,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n",
         "Memory amount limit (incl. children)",
         operatornames[q->operator], q->limit,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     case RESOURCE_ID_TOTAL_MEM_PERCENT:
@@ -1056,8 +1073,8 @@ void Util_printService(Service_T s) {
         "else if passed %s then %s\n",
         "Memory usage limit (incl. children)",
         operatornames[q->operator], q->limit/10.0,
-        ratio1, actionnames[a->failed->id],
-        ratio2, actionnames[a->passed->id]);
+        ratio1, a->failed->description,
+        ratio2, a->passed->description);
       break;
 
     }    
@@ -1070,8 +1087,11 @@ void Util_printService(Service_T s) {
     EventAction_T a= s->action_TIMEOUT;
     printf(" %-20s = If %d restart within %d cycles then %s "
       "else if passed then %s\n",
-      "Timeout", s->to_start, s->to_cycle, actionnames[a->failed->id],
-      actionnames[a->passed->id]);
+      "Timeout",
+      s->to_start,
+      s->to_cycle,
+      a->failed->description,
+      a->passed->description);
   }
 
   for(r= s->maillist; r; r= r->next) {
@@ -1392,8 +1412,8 @@ char *Util_urlEncode(char *uri) {
 
 
 /**
- * Unescape an url string. The <code>url</code> parameter is modified
- * by this method.
+ * Unescape an url string and remove redundant slashes. The
+ * <code>url</code> parameter is modified by this method.
  * @param url an escaped url string
  * @return A pointer to the unescaped <code>url</code>string
  */
@@ -1408,6 +1428,9 @@ char *Util_urlDecode(char *url) {
       url[x]= x2c(&url[y+1]);
       y+=2;
     }
+    while(url[x] == '/' && url[y+1] == '/') {
+      y++;
+    }
   }
   url[x]= 0;
   return url;
@@ -1450,7 +1473,7 @@ char *Util_getBasicAuthHeader() {
     return auth;
   }
 
-  LogError("Cleattext credentials needed for basic authorization!\n");
+  LogError("Cleartext credentials needed for basic authorization!\n");
   return NULL;
 
 }
@@ -1712,7 +1735,8 @@ process_partial_block:
  * Reset the service information structure
  */
 void Util_resetInfo(Service_T s) {
-  FREE(s->inf->cs_sum);
+  if(s->inf->cs_sum)
+    FREE(s->inf->cs_sum);
   memset(s->inf, 0, sizeof *(s->inf));
   s->inf->_pid=        -1;
   s->inf->_ppid=       -1;
@@ -1941,7 +1965,9 @@ static char x2c(char *hex) {
  * Print registered events list
  */
 static void printevents(unsigned int events) {
-  if(events == (~((unsigned int)0))) {
+  if(events == EVENT_NULL) {
+    printf("No events");
+  } else if(events == EVENT_ALL) {
     printf("All events");
   } else {
     if(IS_EVENT_SET(events, EVENT_CHANGED))
@@ -1982,4 +2008,3 @@ static void printevents(unsigned int events) {
   printf("\n");
 }
 
-
