@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2006 by the monit project group.
+ * Copyright (C), 2000-2007 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -80,6 +80,7 @@
 #include "net.h"
 #include "device.h"
 #include "process.h"
+#include "protocol.h"
 
 
 /**
@@ -90,7 +91,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: validate.c,v 1.157 2006/07/19 17:33:37 martinp Exp $
+ *  @version \$Id: validate.c,v 1.163 2007/01/29 18:10:32 martinp Exp $
  *  @file
  */
 
@@ -172,7 +173,9 @@ void validate() {
         s->doaction = ACTION_IGNORE;
       } else if(s->monitor && !check_skip(s) && !check_timeout(s)) {
         s->check(s);
-        s->monitor= MONITOR_YES;
+        /* The monitoring may be disabled by some matching rule in s->check
+         * so we have to check again before setting to MONITOR_YES */
+        if(s->monitor != MONITOR_NOT) s->monitor= MONITOR_YES;
       }
       time(&s->collected);
       unset_signal_block(&os);
@@ -576,20 +579,33 @@ static void check_connection(Service_T s, Port_T p) {
   socket= socket_create(p);
   if(!socket) {
     snprintf(report, STRLEN,
-      "'%s' failed, cannot open a connection to %s via %s",
-      s->name, p->address, Util_portTypeDescription(p));
+      "'%s' failed, cannot open a connection to %s%s%s",
+      s->name, p->address,
+      p->family==AF_INET?" via ":"",
+      p->family==AF_INET?Util_portTypeDescription(p):"");
     rv= FALSE;
     goto error;
   } else {
-    DEBUG("'%s' succeeded connecting to %s via %s\n",
-      s->name, p->address, Util_portTypeDescription(p));
+    DEBUG("'%s' succeeded connecting to %s%s%s\n",
+      s->name, p->address,
+      p->family==AF_INET?" via ":"",
+      p->family==AF_INET?Util_portTypeDescription(p):"");
   }
 
-  /* Verify that the socket is ready for i|o */
-  if(! socket_is_ready(socket)) {
+  /* Verify that the socket is ready for i|o. TCP sockets are checked anytime, UDP
+   * sockets just when there is no specific protocol test used since the socket_is_ready()
+   * adds 2s delay when used with UDP socket. When there is specific protocol used, we
+   * don't need it for UDP, since the protocol test is sufficient */
+  if((socket_get_type(socket) != SOCK_DGRAM || p->protocol->check == check_default)
+       &&
+     !socket_is_ready(socket))
+  {
     snprintf(report, STRLEN,
-      "'%s' failed, the socket at %s via %s is not ready for i|o -- %s",
-      s->name, p->address, Util_portTypeDescription(p), STRERROR);
+      "'%s' failed, the socket at %s%s%s is not ready for i|o -- %s",
+      s->name, p->address,
+      p->family==AF_INET?" via ":"",
+      p->family==AF_INET?Util_portTypeDescription(p):"",
+      STRERROR);
     rv= FALSE;
     goto error;
   }
@@ -597,13 +613,17 @@ static void check_connection(Service_T s, Port_T p) {
   /* Run the protocol verification routine through the socket */
   if(! p->protocol->check(socket)) {
     snprintf(report, STRLEN,
-      "'%s' failed protocol test [%s] at %s via %s",
-      s->name, p->protocol->name, p->address, Util_portTypeDescription(p));
+      "'%s' failed protocol test [%s] at %s%s%s",
+      s->name, p->protocol->name, p->address,
+      p->family==AF_INET?" via ":"",
+      p->family==AF_INET?Util_portTypeDescription(p):"");
     rv= FALSE;
     goto error;
   } else {
-    DEBUG("'%s' succeeded testing protocol [%s] at %s via %s\n",
-      s->name, p->protocol->name, p->address, Util_portTypeDescription(p));
+    DEBUG("'%s' succeeded testing protocol [%s] at %s%s%s\n",
+      s->name, p->protocol->name, p->address,
+      p->family==AF_INET?" via ":"",
+      p->family==AF_INET?Util_portTypeDescription(p):"");
   }
 
   /* Get time of connection attempt finish */
@@ -622,8 +642,10 @@ static void check_connection(Service_T s, Port_T p) {
   } else {
     p->is_available= TRUE;
     Event_post(s, EVENT_CONNECTION, STATE_PASSED, p->action,
-      "'%s' connection passed to %s via %s", s->name,
-      p->address, Util_portTypeDescription(p));
+      "'%s' connection passed to %s%s%s", s->name,
+      p->address,
+      p->family==AF_INET?" via ":"",
+      p->family==AF_INET?Util_portTypeDescription(p):"");
   }
       
 }
@@ -924,7 +946,8 @@ static void check_checksum(Service_T s) {
 
   cs= s->checksum;
 
-  FREE(s->inf->cs_sum);
+  if(s->inf->cs_sum)
+    FREE(s->inf->cs_sum);
   s->inf->cs_sum= Util_getChecksum(s->path, cs->type);
 
   if(s->inf->cs_sum) {
@@ -1498,15 +1521,15 @@ static int check_skip(Service_T s) {
 
   ASSERT(s);
   
-  if(!s->def_every)
-    return FALSE;
-  
   if(s->visited) {
     DEBUG("'%s' check skipped -- service already handled "
           "in a dependency chain\n", s->name);
     return TRUE;
   }
 
+  if(!s->def_every)
+    return FALSE;
+  
   if(++s->nevery < s->every)
     return TRUE;
 
